<?php


namespace PB\WithdrawalQueue\Admin\Controller;


use PB\WithdrawalQueue\Addon;
use PB\WithdrawalQueue\Entity\PaymentSystem;
use XF;
use XF\Admin\Controller\AbstractController;
use XF\ControllerPlugin\Delete;
use XF\ControllerPlugin\Sort;
use XF\ControllerPlugin\Toggle;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Repository;
use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\Exception;
use XF\Mvc\Reply\Redirect;
use XF\Mvc\Reply\View;
use XF\PrintableException;

class PaymentSystems extends AbstractController
{
	public function actionIndex()
	{
		$viewParams = [
			'paymentSystems' => $this->getPaymentSystemRepo()->findPaymentSystems()->fetch()
		];

		return $this->view(Addon::shortName('PaymentSystems\Listing'), Addon::prefix('payment_system_list'), $viewParams);
	}

	public function paymentSystemAddEdit(PaymentSystem $paymentSystem)
	{
		$viewParams = [
			'paymentSystem' => $paymentSystem
		];

		return $this->view(Addon::shortName('PaymentSystems\Edit'), Addon::prefix('payment_system_edit'), $viewParams);
	}

	public function actionAdd()
	{
		/** @var PaymentSystem $paymentSystem */
		$paymentSystem = $this->em()->create(Addon::shortName('PaymentSystem'));
		return $this->paymentSystemAddEdit($paymentSystem);
	}

	/**
	 * @param ParameterBag $params
	 * @return View
	 * @throws Exception
	 */
	public function actionEdit(ParameterBag $params)
	{
		/** @var PaymentSystem $paymentSystem */
		$paymentSystem = $this->assertPaymentSystemExists($params->system_id);
		return $this->paymentSystemAddEdit($paymentSystem);
	}

	protected function paymentSystemSaveProcess(PaymentSystem $paymentSystem)
	{
		$input = $this->filter([
			'title' => 'str',
			'description' => 'str',
			'input_type' => 'str',
			'tax' => 'float',
			'percent' => 'float',
			'min_amount' => 'float',
			'step_amount' => 'float',
			'decimal_places' => 'uint',
			'payout_currency' => 'str',

			'currency_id' => 'uint',
			'currency_rate' => 'float',

			'display_order' => 'uint',
			'active' => 'bool',
			'extra_data' => 'array',
		]);

		$form = $this->formAction();

		/** @var \PB\WithdrawalQueue\Service\PaymentSystem\Icon $imageService */
		$imageService = $this->service(Addon::shortName('PaymentSystem\Icon'), $paymentSystem);

		$form->validate(function (FormAction $form) use ($input, $imageService) {
			if ($input['title'] === '')
			{
				$form->logError(XF::phrase('please_enter_valid_title'), 'title');
			}
			$upload = $this->request->getFile('image', false, false);
			if ($upload && !$imageService->setImageFromUpload($upload))
			{
				$form->logError($imageService->getError());
			}
		});

		$form->basicEntitySave($paymentSystem,
			$input + [
				'currency_addon' => $this->options()->pb_wq_currencyAddon
			]
		);

		$form->complete(function () use ($imageService) {
			try
			{
				$imageService->updateImage();
			}
			catch (\LogicException $e)
			{
				$this->error($e);
			}
		});

		return $form;
	}

	/**
	 * @param ParameterBag $params
	 * @return Redirect
	 * @throws Exception
	 * @throws PrintableException
	 */
	public function actionSave(ParameterBag $params)
	{
		$this->assertPostOnly();

		if ($params->system_id)
		{
			/** @var PaymentSystem $paymentSystem */
			$paymentSystem = $this->assertPaymentSystemExists($params->system_id);
		}
		else
		{
			/** @var PaymentSystem $paymentSystem */
			$paymentSystem = $this->em()->create(Addon::shortName('PaymentSystem'));
		}

		$this->paymentSystemSaveProcess($paymentSystem)->run();

		$redirect = $this->buildLink( 'withdraw/payment-systems/' . $this->buildLinkHash($paymentSystem->getEntityId()));
		return $this->redirect($redirect);
	}

	/**
	 * @param ParameterBag $params
	 * @return Redirect|View
	 * @throws Exception
	 */
	public function actionDelete(ParameterBag $params)
	{
		/** @var PaymentSystem $entity */
		$entity = $this->assertPaymentSystemExists($params->system_id);

		/** @var Delete $deletePlugin */
		$deletePlugin = $this->plugin('XF:Delete');
		return $deletePlugin->actionDelete(
			$entity,
			$this->buildLink('withdraw/payment-systems/delete', $entity),
			$this->buildLink('withdraw/payment-systems', $entity),
			$this->buildLink('withdraw/payment-systems'),
			$entity->title
		);
	}

	public function actionToggle()
	{
		/** @var Toggle $plugin */
		$plugin = $this->plugin('XF:Toggle');
		return $plugin->actionToggle(Addon::shortName('PaymentSystem'));
	}

	/**
	 * @return Redirect|View
	 */
	public function actionSort()
	{
		$paymentSystems = $this->getPaymentSystemRepo()->findPaymentSystems()->fetch();

		if ($this->isPost())
		{
			/** @var Sort $sorter */
			$sorter = $this->plugin('XF:Sort');
			$sorter->sortFlat($this->filter('contents', 'json-array'), $paymentSystems);

			return $this->redirect($this->buildLink('withdraw/payment-systems'));
		}

		$viewParams = [
			'contents' => $paymentSystems
		];

		return $this->view(Addon::shortName('PaymentSystems\Sort'), Addon::prefix('payment_system_sort'), $viewParams);
	}

	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('pb_wq_manage');
	}

	/**
	 * @param $id
	 * @param null $with
	 * @param null $phraseKey
	 * @return Entity|\PB\WithdrawalQueue\Repository\PaymentSystem
	 * @throws Exception
	 */
	protected function assertPaymentSystemExists($id, $with = null, $phraseKey = null)
	{
		return $this->assertRecordExists(Addon::shortName('PaymentSystem'), $id, $with, $phraseKey);
	}

	/**
	 * @return Repository|\PB\WithdrawalQueue\Repository\PaymentSystem
	 */
	protected function getPaymentSystemRepo()
	{
		return $this->repository(Addon::shortName('PaymentSystem'));
	}
}