<?php


namespace PB\WithdrawalQueue\Admin\Controller;


use PB\WithdrawalQueue\Addon;
use XF;
use XF\Admin\Controller\AbstractController;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Repository;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\Exception;

class Withdraws extends AbstractController
{
	public function actionIndex()
	{
		$withdrawRepo = $this->getWithdrawRepo();
		$withdrawFinder = $withdrawRepo->findWithdraws();

		$this->applySearchFilters($withdrawFinder);
		$filters = $this->getSearchFilterInput(true);

		if ($filters && $this->isPost())
		{
			return $this->redirect($this->buildLink('withdraw', null, $filters), '');
		}

		$page = $this->filterPage();
		$perPage = $this->options()->pb_wq_withdrawRop;
		$withdrawFinder->limitByPage($page, $perPage);
		$total = $withdrawFinder->total();

		$withdraws = $withdrawFinder->fetch();

		$this->assertValidPage($page, $perPage, $total, 'withdraw');

		$viewParams = [
			'withdraws' => $withdraws,
			'total' => $total,
			'page' => $page,
			'perPage' => $perPage,
			'conditions' => $filters
		];

		return $this->view(Addon::shortName('Withdraws\Listing'), Addon::prefix('withdraw_list'), $viewParams);
	}

	public function getAvailableSorts()
	{
		return [
			'update_date' => \XF::phrase('pb_wq_update_date')
		];
	}

	public function actionRefineSearch()
	{
		/** @var \PB\WithdrawalQueue\Repository\PaymentSystem $paymentRepo */
		$paymentRepo = $this->repository(Addon::shortName('PaymentSystem'));
		$paymentSystems = $paymentRepo->findPaymentSystems()->fetch();

		$viewParams = [
			'sorts' => $this->getAvailableSorts(),
			'conditions' => $this->getSearchFilterInput(),
			'datePresets' => \XF::language()->getDatePresets(),
			'paymentSystems' => $paymentSystems
		];

		return $this->view(Addon::shortName('Withdraws\RefineSearch'), Addon::prefix('withdraw_refine_search'), $viewParams);
	}

	private function applySearchFilters(\PB\WithdrawalQueue\Finder\Withdraw $finder)
	{
		$filters = $this->getSearchFilterInput();

		if ($filters['username'])
		{
			$finder->where('User.username', '=', $filters['username']);
		}

		if ($filters['sum_min'])
		{
			$finder->where('sum', '>=', $filters['sum_min']);
		}
		if ($filters['sum_max'])
		{
			$finder->where('sum', '<=', $filters['sum_max']);
		}

		if ($filters['system_ids'])
		{
			$finder->where('system_id', $filters['system_ids']);
		}

		if ($filters['withdraw_states'])
		{
			$finder->where('withdraw_state', $filters['withdraw_states']);
		}

		if ($filters['start'])
		{
			$finder->where('create_date', '>', $filters['start']);
		}
		if ($filters['end'])
		{
			$finder->where('create_date', '<', $filters['end']);
		}

		if ($filters['sort'] && $filters['order'])
		{
			if (isset($this->getAvailableSorts()[$filters['sort']]))
			{
				$sort = $filters['sort'];
			}
			else
			{
				$sort = 'create_date';
			}

			$finder->order([
				[$sort, (strcasecmp($filters['order'], 'ASC') == 0) ? 'ASC' : 'DESC'],
				['withdraw_id', 'ASC']
			]);
		}
	}

	private function getSearchFilterInput($removeEmpty = false): array
	{
		$input = $this->filter([
			'username' => 'str',
			'sum_min' => 'float',
			'sum_max' => 'float',
			'system_ids' => 'array-uint',
			'withdraw_states' => 'array-str',
			'display_options' => 'array',
			'start' => 'datetime',
			'end' => 'datetime',

			'sort' => 'str',
			'order' => 'str',
		]);

		if ($removeEmpty)
		{
			$input = array_filter($input);
		}

		return $input;
	}

	public function actionExport()
	{
		$withdrawRepo = $this->getWithdrawRepo();
		$withdrawFinder = $withdrawRepo->findWithdraws();

		$filters = $this->getSearchFilterInput(true);
		$this->applySearchFilters($withdrawFinder);

		if ($this->isPost())
		{
			$data = $withdrawFinder->fetch();

			$exportData = [];

			/** @var \PB\WithdrawalQueue\Entity\Withdraw $value */
			foreach ($data as $value)
			{
				$exportData[] = $value->getCsvExportData();
			}

			/** @var \PB\WithdrawalQueue\Service\Withdraw\Export $exportService */
			$exportService = $this->service('PB\WithdrawalQueue:Withdraw\Export', $exportData);

			$phpVersion = phpversion();
			if (version_compare($phpVersion, '7.4.0', '<'))
			{
				throw $this->errorException(\XF::phrase('pb_wq_csv_export_php_min_x_version', ['version' => '7.4.0']));
			}

			$csv = $exportService->exportToCsv();
			$viewParams = ['csv' => $csv->getContent()];
			$this->setResponseType('raw');

			return $this->view(Addon::shortName('Withdraws\Export'), '', $viewParams);
		}

		$viewParams = [
			'conditions' => $filters,
			'total' => $withdrawFinder->total()
		];

		return $this->view(Addon::shortName('Withdraws\Export'), Addon::prefix('withdraw_export'), $viewParams);
	}

	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('pb_wq_manage');
	}

	/**
	 * @param $id
	 * @param null $with
	 * @param null $phraseKey
	 * @return Entity|\PB\WithdrawalQueue\Repository\PaymentSystem
	 * @throws Exception
	 */
	protected function assertWithdrawExists($id, $with = null, $phraseKey = null)
	{
		return $this->assertRecordExists(Addon::shortName('Withdraw'), $id, $with, $phraseKey);
	}

	/**
	 * @return Repository|\PB\WithdrawalQueue\Repository\Withdraw
	 */
	protected function getWithdrawRepo()
	{
		return $this->repository(Addon::shortName('Withdraw'));
	}
}