<?php

namespace PB\WithdrawalQueue\Api\Controller;

use PB\WithdrawalQueue\Addon;
use PB\WithdrawalQueue\Entity\Requisite as RequisiteEntity;
use XF\Mvc\Entity\Entity;
use XF\Mvc\ParameterBag;

class Requisite extends \XF\Api\Controller\AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertApiScopeByRequestMethod('pb_wq_requisite');
	}

	/**
	 * @api-desc Gets information about the specified requisite.
	 *
	 * @api-out PB_WithdrawalQueue_Requisite $requisite
	 */
	public function actionGet(ParameterBag $params)
	{
		$requisite = $this->assertViewableRequisite($params->requisite_id);

		$result = [
			'requisite' => $requisite->toApiResult(Entity::VERBOSITY_VERBOSE)
		];

		return $this->apiResult($result);
	}

	protected function setupRequisiteEdit(RequisiteEntity $requisite)
	{
		/** @var \PB\WithdrawalQueue\Service\Requisite\Editor $editor */
		$editor = $this->service(Addon::shortName('Requisite\Editor'), $requisite);

		$wallet = $this->filter('wallet', 'str');
		if ($wallet)
		{
			$editor->setWallet($wallet);
		}

		$customFields = $this->filter('custom_fields', 'array');
		if ($customFields)
		{
			$editor->setCustomFields($customFields);
		}

		return $editor;
	}

	protected function finalizeRequisiteEdit(\PB\WithdrawalQueue\Service\Requisite\Editor $editor)
	{
	}

	/**
	 * @api-desc Updates the specified requisite
	 *
	 * @api-in str $wallet
	 * @api-in string $custom_fields[<name>]
	 *
	 * @api-out true $success
	 * @api-out PB_WithdrawalQueue_Requisite $requisite
	 */
	public function actionPost(ParameterBag $params)
	{
		$requisite = $this->assertViewableRequisite($params->requisite_id);

		if (\XF::isApiCheckingPermissions() && !$requisite->canEdit($error))
		{
			return $this->noPermission($error);
		}

		$editor = $this->setupRequisiteEdit($requisite);
		if (!$editor->validate($errors))
		{
			return $this->error($errors);
		}

		$editor->save();
		$this->finalizeRequisiteEdit($editor);

		return $this->apiSuccess([
			'requisite' => $requisite->toApiResult(Entity::VERBOSITY_VERBOSE)
		]);
	}

	/**
	 * @api-desc Deletes the specified requisite.
	 *
	 * @api-out true $success
	 *
	 * @param ParameterBag $params
	 * @return \XF\Api\Mvc\Reply\ApiResult|\XF\Mvc\Reply\AbstractReply|\XF\Mvc\Reply\Error
	 * @throws \XF\Mvc\Reply\Exception
	 * @throws \XF\PrintableException
	 */
	public function actionDelete(ParameterBag $params)
	{
		$requisite = $this->assertViewableRequisite($params->requisite_id);

		if (\XF::isApiCheckingPermissions() && !$requisite->canDelete($error))
		{
			return $this->noPermission($error);
		}

		if (!$requisite->preDelete())
		{
			return $this->error($requisite->getErrors());
		}

		$requisite->delete();

		return $this->apiSuccess();
	}

	/**
	 * @param int $id
	 * @param string|array $with
	 *
	 * @return RequisiteEntity
	 *
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertViewableRequisite($id, $with = 'api')
	{
		$visitor = \XF::visitor();

		/** @var \PB\WithdrawalQueue\Finder\Requisite $finder */
		$finder = $this->finder(Addon::shortName('Requisite'));
		$finder->forUser($visitor);
		$finder->where('requisite_id', $id);
		$finder->with($with);

		/** @var RequisiteEntity $requisite */
		$requisite = $finder->fetchOne();
		if (!$requisite)
		{
			throw $this->exception($this->notFound(Addon::phrase('requested_requisite_not_found')));
		}

		return $requisite;
	}
}