<?php


namespace PB\WithdrawalQueue\CurrencyHandler;


use DBTech\Credits\Repository\Currency;
use PB\WithdrawalQueue\Addon;
use XF;
use XF\Entity\User;
use XF\Mvc\Entity\Entity;

class DBTechCredits extends AbstractHandler
{
	/**
	 * @var \DBTech\Credits\Entity\Currency|null
	 */
	protected $currency;

	protected function getCurrency($currencyId)
	{
		if ($this->currency)
		{
			return $this->currency;
		}

		$app = XF::app();
		$container = $app->container();
		if (isset($container['dbtechCredits.currencies']))
		{
			$currencies = $container['dbtechCredits.currencies'] ?? null;
			if ($currencies)
			{
				/** @var \DBTech\Credits\Entity\Currency $currency */
				$currency = $currencies[$currencyId] ?? null;
				$this->currency = $currency;

				return $currency;
			}
		}

		return null;
	}

	public function getCurrencyTitle($currencyId)
	{
		$currency = $this->getCurrency($currencyId);
		if ($currency)
		{
			return $currency->title;
		}

		return '';
	}

	public function getFormattedAmount($amount, $currencyId)
	{
		$currency = $this->getCurrency($currencyId);
		if ($currency)
		{
			return $currency->prefix . $currency->getFormattedValue($amount) . $currency->suffix . " " . $currency->title;
		}

		return $amount;
	}

	public function getChargedAmount($amount, $currencyId)
	{
		$currency = $this->getCurrency($currencyId);
		if ($currency)
		{
			/** @var \DBTech\Credits\Repository\EventTrigger $adjust */
			$eventTriggerRepo = XF::repository('DBTech\Credits:EventTrigger');

			$user = \XF::visitor();

			/** @var \DBTech\Credits\EventTrigger\Adjust $adjustHandler */
			$adjustHandler = $eventTriggerRepo->getHandler('adjust');
			$extraParams = $adjustHandler->initExtraParams($user, false, [
				'currency_id' => $currencyId,
				'multiplier' => -$amount,
			]);

			$events = $adjustHandler->getApplicableEvents($extraParams, $user, null);
			if ($events)
			{
				/** @var \DBTech\Credits\Entity\Event $event */
				$event = $events->first();
				if ($event)
				{
					$amount = $event->getCalculatedAmount($adjustHandler, $user, $extraParams);
				}
			}
		}
		return $amount;
	}

	public function getCurrencyOptionsData()
	{
		$choices = [
			0 => ['_type' => 'option', 'value' => 0, 'label' => XF::phrase('(none)')]
		];

		/** @var Currency $currencyRepo */
		$currencyRepo = XF::repository('DBTech\Credits:Currency');
		$currencies = $currencyRepo->getCurrencyTitlePairs();

		foreach ($currencies as $currencyId => $label)
		{
			$choices[$currencyId] = [
				'value' => $currencyId,
				'label' => $label
			];
		}

		return $choices;
	}

	public function testCharge(Entity $content, User $user, $amount, $currencyId, &$errors = []): bool
	{
		/** @var \DBTech\Credits\XF\Entity\User $user */
		$currency = $this->getCurrency($currencyId);

		if (!$currency)
		{
			$errors[] = Addon::phrase('no_currencies_available');
			return false;
		}

		try
		{
			$userCurrency = $user->getDbtechCreditsCurrency($currency);
		}
		catch (\LogicException $exception)
		{
			$errors[] = $exception->getMessage();

			return false;
		}

		if ($amount > $userCurrency)
		{
			$errors[] = Addon::phrase('credits_cancel_price_charge', [
				'amount' => XF::language()->numberFormat($amount, $currency->decimals),
				'currency' => $currency->title
			]);

			return false;
		}

		if ($amount > 2147483647)
		{
			$errors[] = XF::phrase('please_enter_number_that_is_no_more_than_x', [
				'max' => 2147483647,
			]);

			return false;
		}

		return true;
	}

	public function charge(Entity $content, User $user, $amount, $currencyId)
	{
		/** @var \DBTech\Credits\Repository\EventTrigger $adjust */
		$eventTriggerRepo = XF::repository('DBTech\Credits:EventTrigger');

		/** @var \DBTech\Credits\EventTrigger\Adjust $adjustHandler */
		$adjustHandler = $eventTriggerRepo->getHandler('adjust');
		$adjustHandler->apply($user->user_id, [
			'currency_id' => $currencyId,
			'multiplier' => -$amount,
			'source_user_id' => $user->user_id,
			'content_type' => $content->getEntityContentType(),
			'content_id' => $content->getEntityId(),
			'message' => Addon::phrase('withdraw_request'),
		], $user);
	}

	public function testRefund(Entity $content, User $user, $amount, $currencyId, &$errors = []): bool
	{
		/** @var \DBTech\Credits\XF\Entity\User $user */
		$currency = $this->getCurrency($currencyId);

		if (!$currency)
		{
			$errors[] = Addon::phrase('no_currencies_available');
			return false;
		}

		try
		{
			$user->getDbtechCreditsCurrency($currency);
		}
		catch (\LogicException $exception)
		{
			$errors[] = $exception->getMessage();

			return false;
		}

		return true;
	}

	public function refund(Entity $content, User $user, $amount, $currencyId)
	{
		/** @var \PB\WithdrawalQueue\Entity\Withdraw $content */

		/** @var \DBTech\Credits\Repository\EventTrigger $adjust */
		$eventTriggerRepo = XF::repository('DBTech\Credits:EventTrigger');

		/** @var \DBTech\Credits\EventTrigger\Adjust $adjustHandler */
		$adjustHandler = $eventTriggerRepo->getHandler('adjust');

		$adjustHandler->apply($user->user_id, [
			'currency_id' => $currencyId,
			'multiplier' => $amount,
			'source_user_id' => $user->user_id,
			'content_type' => $content->getEntityContentType(),
			'content_id' => $content->getEntityId(),
			'message' => Addon::phrase('withdraw_refund'),
		], $user);
	}
}
