<?php


namespace PB\WithdrawalQueue\Entity;


use PB\WithdrawalQueue\Addon;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;
use XF\Util\File;

/**
 * Class PaymentSystem
 * @package PB\WithdrawalQueue\Entity
 *
 * @property int $system_id
 * @property string $title
 * @property string $description
 * @property string $input_type
 * @property float $currency_rate
 * @property float $tax
 * @property float $percent
 * @property float $min_amount
 * @property int $step_amount
 * @property int $decimal_places
 * @property int $image_date
 * @property int $display_order
 * @property bool $active
 * @property array $field_cache
 * @property array $extra_data
 *
 * GETTERS
 * @property string $image_url
 * @property \PB\WithdrawalQueue\WalletType\AbstractHandler|null wallet_type_handler
 */
class PaymentSystem extends Entity
{
	use CurrencyHandlerTrait;

	public function getStepMinAmount()
	{
		$decimalPlaces = $this->decimal_places;
		return number_format("0." . str_pad(1, $decimalPlaces, '0', STR_PAD_LEFT), $decimalPlaces);
	}

	public function getAllowedInputTypes()
	{
		return [
			'text' => ['title' => \XF::phrase('text')],
			'tel' => ['title' => \XF::phrase('tel')],
			'email' => ['title' => \XF::phrase('email'), 'handler_class' => Addon::shortName('Email')],
			'bitcoin' => ['title' => Addon::phrase('wallet_type_bitcoin'), 'handler_class' => Addon::shortName('Bitcoin')],
		];
	}

	/**
	 * @return mixed|null|\PB\WithdrawalQueue\WalletType\AbstractHandler
	 * @throws \Exception
	 */
	public function getWalletTypeHandler()
	{
		$inputTypes = $this->getAllowedInputTypes();
		if (!isset($inputTypes[$this->input_type]['handler_class']))
		{
			return null;
		}

		$handlerClass = $inputTypes[$this->input_type]['handler_class'];
		$class = \XF::stringToClass($handlerClass, '%s\WalletType\%s');
		if (!class_exists($class))
		{
			return null;
		}

		$class = \XF::extendClass($class);
		return new $class($this);
	}

	public function getExtraDataValue($key)
	{
		return $this->extra_data[$key] ?? null;
	}

	public function getActualPayoutAmount(float $amount, $format = false)
	{
		$percentage = $this->percent;

		if ($percentage > 0)
		{
			$amount = $amount - ($percentage / 100) * $amount;
		}

		$amount = $amount - $this->tax;

		if ($format)
		{
			return number_format($amount, $this->decimal_places);
		}

		return $amount;
	}

	public function getExchangedChargeAmount(float $amount)
	{
		return $amount * $this->currency_rate;
	}

	public function getActualChargeAmount(float $amount, $format = false)
	{
		$amount = $this->getExchangedChargeAmount($amount);

		$currencyHandler = $this->getCurrencyHandler();
		if ($currencyHandler)
		{
			$chargedAmount = $currencyHandler->getChargedAmount($amount, $this->currency_id);
			if ($format)
			{
				return $currencyHandler->getFormattedAmount($chargedAmount, $this->currency_id);
			}

			return $chargedAmount;
		}

		return $amount;
	}

	public function getNewRequisite()
	{
		/** @var Requisite $requisite */
		$requisite = $this->em()->create(Addon::shortName('Requisite'));
		$requisite->system_id = $this->system_id;

		return $requisite;
	}

	// IMAGES

	public function getImageAbstractedPath($code, $getExistingValue = false)
	{
		$path = $this->getImagePath($code, $getExistingValue);
		if (empty($path))
		{
			return null;
		}

		return 'data://' . $path;
	}

	/**
	 * @param $code
	 * @param false $getExistingValue
	 * @return string|null
	 */
	public function getImagePath($code, $getExistingValue = false)
	{
		$code = strtolower($code);
		$sizeMap = $this->getImageSizeMap();
		if (!isset($sizeMap[$code]))
		{
			$code = 'l';
		}

		$imageDate = $getExistingValue ? $this->getExistingValue('image_date') : $this->image_date;

		if (!$imageDate)
		{
			return "pb/withdrawal_queue/none_$code.jpg";
		}

		return sprintf('pb/withdrawal_queue/%s_%d_%s.jpg', $this->system_id, $imageDate, $code);
	}

	public function getImageSizeMap()
	{
		return [
			'l' => $this->getImageSizePixels('L'),
			't' => $this->getImageSizePixels('T'),
		];
	}

	public function getImageSizePixels($code)
	{
		$code = strtoupper($code);
		switch ($code)
		{
			case 'L':
				return -1;
			case 'T':
				return 22;
		}

		return 0;
	}

	public function getImageUrl($code = 'L', $canonical = false)
	{
		$app = $this->app();
		$path = $this->getImagePath($code);
		return $app->applyExternalDataUrl($path, $canonical);
	}

	public function deleteExistingImages()
	{
		if (!$this->image_date)
		{
			return;
		}

		$sizeMap = $this->getImageSizeMap();
		foreach ($sizeMap as $code => $size)
		{
			$dataFile = $this->getImageAbstractedPath($code, true);
			if (is_string($dataFile) && $dataFile != "pb/withdrawal_queue/none_$code.jpg")
			{
				File::deleteFromAbstractedPath($dataFile);
			}
		}
	}

	protected function setupApiResultData(\XF\Api\Result\EntityResult $result, $verbosity = self::VERBOSITY_NORMAL, array $options = [])
	{
	}

	// Lifecycle

	protected function _preSave()
	{
		if ($this->currency_rate <= 0)
		{
			$this->error(Addon::phrase('currency_rate_cannot_be_lower_or_equal_0'));
		}
	}

	protected function _postDelete()
	{
		$this->deleteExistingImages();
	}

	public static function getStructure(Structure $structure)
	{
		$structure->table = 'xf_pb_wq_payment_system';
		$structure->shortName = Addon::shortName('PaymentSystem');
		$structure->primaryKey = 'system_id';

		$structure->columns = [
			'system_id' => ['type' => self::UINT, 'autoIncrement' => true],
			'title' => ['type' => self::STR, 'required' => true, 'maxLength' => 255, 'api' => true],
			'description' => ['type' => self::STR, 'default' => '', 'api' => true],
			'input_type' => ['type' => self::STR, 'default' => 'text', 'api' => true],
			'tax' => ['type' => self::FLOAT, 'required' => false, 'default' => 0, 'api' => true],
			'percent' => ['type' => self::FLOAT, 'required' => false, 'default' => 0, 'api' => true],

			'currency_rate' => ['type' => self::FLOAT, 'default' => 1, 'api' => true],

			'min_amount' => ['type' => self::FLOAT, 'required' => false, 'default' => 0.0, 'min' => 0, 'api' => true],
			'step_amount' => ['type' => self::FLOAT, 'required' => false, 'default' => 1.0, 'api' => true],
			'decimal_places' => ['type' => self::UINT, 'required' => false, 'default' => 8, 'min' => 0, 'max' => 8, 'api' => true],
			'display_order' => ['type' => self::UINT, 'default' => 0, 'api' => true],
			'image_date' => ['type' => self::UINT, 'default' => 0, 'api' => true],
			'active' => ['type' => self::BOOL, 'default' => true, 'api' => true],
			'field_cache' => ['type' => self::JSON_ARRAY, 'default' => [], 'api' => true],
			'extra_data' => ['type' => self::JSON_ARRAY, 'default' => [], 'api' => true],
		];

		$structure->getters = [
			'wallet_type_handler' => true,
			'image_url' => true
		];

		self::addCurrencyHandlerStructureElements($structure);

		return $structure;
	}

	protected function _setupDefaults()
	{
		$this->active = true;
		$this->display_order = 10;
		$this->currency_addon = $this->app()->options()->pb_wq_currencyAddon;
		$this->extra_data = ['input_type' => 'text'];
	}
}
