<?php


namespace PB\WithdrawalQueue\Entity;


use PB\WithdrawalQueue\Addon;
use XF;
use XF\Entity\User;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * Class Requisite
 * @package PB\WithdrawalQueue\Entity
 *
 * COLUMNS
 * @property int $requisite_id
 * @property int $system_id
 * @property int $user_id
 * @property string $wallet
 * @property int $create_date
 * @property int $update_date
 * @property array $custom_fields_
 *
 * GETTERS
 * @property \XF\CustomField\Set $custom_fields
 *
 * RELATIONS
 * @property User $User
 * @property PaymentSystem $PaymentSystem
 */
class Requisite extends Entity implements XF\Entity\LinkableInterface
{
	public function canView(&$error = null)
	{
		return $this->user_id === XF::visitor()->user_id;
	}

	public function canEdit(&$error = null)
	{
		return $this->user_id === XF::visitor()->user_id;
	}

	public function canDelete(&$error = null)
	{
		return $this->user_id === XF::visitor()->user_id;
	}

	public function getContentUrl(bool $canonical = false, array $extraParams = [], $hash = null)
	{
		$route = $canonical ? 'canonical:withdraw/requisite/edit' : 'withdraw/requisite/edit';
		return $this->app()->router('public')->buildLink($route, $this, $extraParams, $hash);
	}

	public function getContentPublicRoute()
	{
		return 'withdraw/requisite';
	}

	public function getContentTitle(string $context = '')
	{
		return $this->PaymentSystem ? $this->PaymentSystem->title : '';
	}

	public function getInputTemplateName()
	{
		if ($this->PaymentSystem && $this->PaymentSystem->wallet_type_handler)
		{
			return $this->PaymentSystem->wallet_type_handler->getInputTemplateName();
		}

		return 'pb_wq_requisite_wallet_input_template_text';
	}

	public function getInputTemplateData()
	{
		$templateData = ['requisite' => $this];
		if ($this->PaymentSystem && $this->PaymentSystem->wallet_type_handler)
		{
			$templateData += $this->PaymentSystem->wallet_type_handler->getInputTemplateData();
		}

		return $templateData;
	}

	public function validateWallet(&$errors = [])
	{
		if ($this->PaymentSystem && $this->PaymentSystem->wallet_type_handler)
		{
			return $this->PaymentSystem->wallet_type_handler->validate($this->wallet, $errors);
		}

		return true;
	}

	public function renderInputTemplate()
	{
		return $this->app()->templater()->renderTemplate(
			'public:' . $this->getInputTemplateName(),
			$this->getInputTemplateData()
		);
	}

	/**
	 * @return \XF\CustomField\Set
	 * @throws \Exception
	 */
	public function getCustomFields()
	{
		$class = $this->app()->extendClass('XF\CustomField\Set');
		$fieldDefinitions = $this->app()->container('customFields.pb_wq_requisite');

		return new $class($fieldDefinitions, $this);
	}

	public function getFieldEditMode()
	{
		return 'user';
	}

	/**
	 * @param XF\Api\Result\EntityResult $result
	 * @param $verbosity
	 * @param array $options
	 * @return void
	 *
	 * @api-out int $requisite_id
	 * @api-out int $system_id
	 * @api-out int $user_id
	 * @api-out string $custom_fields
	 * @api-out string $wallet
	 * @api-out int $create_date
	 * @api-out int $update_date
	 * @api-out object $custom_fields
	 */
	protected function setupApiResultData(\XF\Api\Result\EntityResult $result, $verbosity = self::VERBOSITY_NORMAL, array $options = [])
	{
		$result->username = $this->User ? $this->User->username : \XF::phrase('unknown_user');

		$result->custom_fields = (object)$this->custom_fields->getNamedFieldValues($this->PaymentSystem->field_cache);
	}

	/************************* LIFE CYCLE ***************************/

	protected function _preSave()
	{
		if ($this->isChanged(['wallet', 'system_id']))
		{
			$this->update_date = XF::$time;
		}
	}

	public static function getStructure(Structure $structure)
	{
		$structure->table = 'xf_pb_wq_requisite';
		$structure->shortName = Addon::shortName('Requisite');
		$structure->primaryKey = 'requisite_id';

		$structure->columns = [
			'requisite_id' => ['type' => self::UINT, 'autoIncrement' => true],
			'system_id' => ['type' => self::UINT, 'required' => true, 'api' => true],
			'user_id' => ['type' => self::UINT, 'required' => true, 'api' => true],
			'wallet' => ['type' => self::STR, 'required' => true, 'maxLength' => 255, 'unique' => true, 'api' => true],
			'create_date' => ['type' => self::UINT, 'default' => XF::$time, 'api' => true],
			'update_date' => ['type' => self::UINT, 'default' => 0, 'api' => true],
			'custom_fields' => ['type' => self::JSON_ARRAY, 'default' => []],
		];

		$structure->getters = [
			'custom_fields' => true,
		];

		$structure->relations = [
			'User' => [
				'entity' => 'XF:User',
				'type' => self::TO_ONE,
				'conditions' => 'user_id',
				'primary' => true
			],
			'PaymentSystem' => [
				'entity' => Addon::shortName('PaymentSystem'),
				'type' => self::TO_ONE,
				'conditions' => 'system_id',
				'primary' => true
			],
		];

		$structure->withAliases = [
			'api' => [
				'User',
				'User.api'
			]
		];

		$structure->behaviors = [
			'XF:CustomFieldsHolder' => [
				'valueTable' => 'xf_pb_wq_requisite_field_value',
			],
		];

		$structure->defaultWith = ['PaymentSystem'];

		return $structure;
	}

	/**
	 * @return XF\Mvc\Entity\Repository|\PB\WithdrawalQueue\Repository\Requisite
	 */
	protected function getRequisiteRepo()
	{
		return $this->repository(Addon::shortName('Requisite'));
	}
}