<?php


namespace PB\WithdrawalQueue\Entity;


use PB\WithdrawalQueue\Addon;
use XF;
use XF\Entity\User;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Repository;
use XF\Mvc\Entity\Structure;

/**
 * Class Withdraw
 * @package PB\WithdrawalQueue\Entity
 *
 * COLUMNS
 * @property int $withdraw_id
 * @property int $user_id
 * @property string $username
 * @property int $update_user_id
 * @property string $update_username
 * @property float $sum
 * @property float $payout
 * @property float $charged
 * @property string $withdraw_state
 * @property string $reject_reason
 * @property int $requisite_id
 * @property string $wallet
 * @property int $system_id
 * @property string $system_title
 * @property int $create_date
 * @property int $update_date
 *
 * GETTERS
 * @property float $formatted_requested_amount
 * @property float $formatted_payout_amount
 * @property string $formatted_charged_amount
 * @property string $state_phrase
 * @property float $commission
 *
 * RELATIONS
 * @property User $User
 * @property User $UpdateUser
 * @property Requisite $Requisite
 * @property PaymentSystem $PaymentSystem
 */
class Withdraw extends XF\Mvc\Entity\Entity
{
	use CurrencyHandlerTrait;

	public function canView(&$error = null)
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = XF::visitor();
		return $visitor->canViewWithdraws($error) && $visitor->user_id == $this->user_id;
	}

	/************************* HELPERS ***************************/

	public function getFormattedRequestedAmount()
	{
		if ($this->PaymentSystem)
		{
			return $this->app()->templater()->func('number', [$this->sum, $this->PaymentSystem->decimal_places]);
		}

		return $this->sum;
	}

	public function getFormattedPayoutAmount()
	{
		if ($this->PaymentSystem)
		{
			return $this->app()->templater()->func('number', [$this->payout, $this->PaymentSystem->decimal_places]);
		}

		return $this->payout;
	}

	public function getFormattedChargedAmount()
	{
		if ($this->charged === null)
		{
			return \XF::phrase('n_a');
		}

		$handler = $this->getCurrencyHandler();
		if ($handler)
		{
			return $handler->getFormattedAmount($this->charged, $this->currency_id);
		}

		return $this->charged;
	}

	/**
	 * @return bool
	 */
	public function hasCommission()
	{
		return $this->payout <> $this->sum;
	}

	/**
	 * @return float
	 */
	public function getCommission()
	{
		if (!$this->hasCommission())
		{
			return 0.00;
		}

		return -floatval($this->sum - $this->payout);
	}

	public function getStatePhrase()
	{
		return Addon::phrase('withdraw_state_' . $this->withdraw_state);
	}

	public function getInputTemplateName()
	{
		if ($inputTemplate = $this->PaymentSystem->getExtraDataValue('input_template'))
		{
			return $inputTemplate;
		}

		return 'pb_wq_requisite_input_text';
	}

	public function getInputTemplateData()
	{
		$params = [
			'requisite' => $this,
			'input_type' => 'text',
		];

		if ($inputType = $this->PaymentSystem->input_type)
		{
			switch ($inputType)
			{
				case 'text':
					$params['input_type'] = 'text';
					break;
				case 'tel':
					$params['input_type'] = 'tel';
					break;
				case 'email':
					$params['input_type'] = 'email';
					break;
			}
		}

		return $params;
	}

	public function renderInputTemplate()
	{
		return $this->app()->templater()->renderTemplate(
			'public:' . $this->getInputTemplateName(),
			$this->getInputTemplateData()
		);
	}

	public function testCharge(&$errors = [])
	{
		$currencyHandler = $this->getCurrencyHandler();
		if ($currencyHandler && $this->User)
		{
			$chargeAmount = $this->PaymentSystem->getExchangedChargeAmount($this->sum);
			return $currencyHandler->testCharge($this, $this->User, $chargeAmount, $this->currency_id, $errors);
		}

		return false;
	}

	public function charge()
	{
		$currencyHandler = $this->getCurrencyHandler();
		if ($currencyHandler && $this->User)
		{
			$chargeAmount = $this->PaymentSystem->getExchangedChargeAmount($this->sum);
			$currencyHandler->charge($this, $this->User, $chargeAmount, $this->currency_id);
			$this->fastUpdate('charged', $chargeAmount);
		}
	}

	public function testRefund(&$errors = [])
	{
		$currencyHandler = $this->getCurrencyHandler();
		if ($currencyHandler && $this->User)
		{
			return $currencyHandler->testRefund($this, $this->User, $this->charged, $this->currency_id, $errors);
		}

		return false;
	}

	public function refund()
	{
		$currencyHandler = $this->getCurrencyHandler();
		if ($currencyHandler && $this->User)
		{
			$currencyHandler->refund($this, $this->User, $this->charged, $this->currency_id);
			$this->fastUpdate('charged', 0);
		}
	}

	public function getCsvExportData()
	{
		$templater = $this->app()->templater();

		return [
			'withdraw_id' => $this->withdraw_id,
			'user_id' => $this->user_id,
			'username' => $this->username,
			'update_user_id' => $this->update_user_id,
			'update_username' => $this->update_username,
			'sum' => $this->sum ? str_replace('.', ',', $this->sum) : 0,
			'payout' => $this->payout ? str_replace('.', ',', $this->payout) : 0,
			'payout_currency' => $this->payout_currency,
			'charged' => $this->charged ? str_replace('.', ',', $this->charged) : 0,
			'withdraw_state' =>  $this->getStatePhrase(),
			'reject_reason' => $this->reject_reason,
			'requisite_id' => $this->requisite_id,
			'wallet' => $this->wallet,
			'system_id' => $this->system_id,
			'system_title' => $this->system_title,

			'create_date' => $templater->func('date_time', [$this->create_date]),
			'update_date' => $templater->func('date_time', [$this->update_date]),
		];
	}

	/**
	 * @param XF\Api\Result\EntityResult $result
	 * @param $verbosity
	 * @param array $options
	 * @return void
	 *
	 * @api-out int $withdraw_id
	 * @api-out int $user_id
	 * @api-out string $username
	 * @api-out int $update_user_id
	 * @api-out string $update_username
	 * @api-out float $sum
	 * @api-out float $payout
	 * @api-out float $charged
	 * @api-out float $commission
	 * @api-out string $withdraw_state
	 * @api-out string $reject_reason
	 * @api-out int $requisite_id
	 * @api-out string $wallet
	 * @api-out int $system_id
	 * @api-out string $system_title
	 * @api-out string $state_phrase
	 * @api-out int $create_date
	 * @api-out int $update_date
	 * @api-out string $formatted_requested_amount
	 * @api-out string $formatted_payout_amount
	 * @api-out string $formatted_charged_amount
	 */
	protected function setupApiResultData(\XF\Api\Result\EntityResult $result, $verbosity = self::VERBOSITY_NORMAL, array $options = [])
	{
		$result->username = $this->User ? $this->User->username : $this->username;
		$result->update_username = $this->User ? $this->User->username : $this->update_username;
		$result->state_phrase = $this->state_phrase;
		$result->commission = $this->commission;
		$result->formatted_requested_amount = $this->formatted_requested_amount;
		$result->formatted_payout_amount = $this->formatted_payout_amount;
		$result->formatted_charged_amount = $this->formatted_charged_amount;
	}

	/************************* LIFE-CYCLE ***************************/

	protected function _preSave()
	{
		if ($this->hasChanges())
		{
			$visitor = XF::visitor();

			$this->update_date = XF::$time;
			$this->update_user_id = $visitor->user_id;
			$this->update_username = $visitor->username;
		}
	}

	protected function _postSave()
	{
		if ($this->isChanged('withdraw_state'))
		{
			$this->rebuildPendingWithdrawCounts();
		}
	}

	protected function _postDelete()
	{
		$this->rebuildPendingWithdrawCounts();
	}

	protected function rebuildPendingWithdrawCounts()
	{
		XF::runOnce('pendingWithdrawsCountsRebuild', function () {
			$this->getWithdrawQueueRepo()->rebuildPendingWithdrawCounts();
		});
	}

	public static function getStructure(Structure $structure)
	{
		$structure->table = 'xf_pb_wq_withdraw';
		$structure->shortName = Addon::shortName('Withdraw');
		$structure->contentType = 'pb_wq_withdraw';
		$structure->primaryKey = 'withdraw_id';
		$structure->columns = [
			'withdraw_id' => ['type' => self::UINT, 'autoIncrement' => true],
			'user_id' => ['type' => self::UINT, 'required' => true, 'api' => true],
			'username' => ['type' => self::STR, 'default' => '', 'maxLength' => 50, 'api' => true],
			'update_user_id' => ['type' => self::UINT, 'default' => 0, 'api' => true],
			'update_username' => ['type' => self::STR, 'default' => '', 'maxLength' => 50, 'api' => true],
			'sum' => ['type' => self::FLOAT, 'required' => true, 'max' => 9999999999.99999999, 'api' => true],
			'payout' => ['type' => self::FLOAT, 'required' => true, 'api' => true],
			'charged' => ['type' => self::FLOAT, 'nullable' => true, 'default' => null, 'api' => true],
			'withdraw_state' => [
				'type' => self::STR, 'allowedValues' => [
					'pending', 'rejected', 'sent'
				],
				'default' => 'pending',
				'api' => true
			],
			'reject_reason' => ['type' => self::STR, 'default' => '', 'maxLength' => 255, 'api' => true],
			'requisite_id' => ['type' => self::UINT, 'default' => 0, 'api' => true],
			'wallet' => ['type' => self::STR, 'required' => true, 'maxLength' => 255, 'api' => true],
			'system_id' => ['type' => self::UINT, 'required' => true, 'api' => true],
			'system_title' => ['type' => self::STR, 'required' => true, 'maxLength' => 255, 'api' => true],

			'create_date' => ['type' => self::UINT, 'default' => XF::$time, 'api' => true],
			'update_date' => ['type' => self::UINT, 'default' => 0, 'api' => true],
		];

		$structure->getters = [
			'state_phrase' => ['getter' => true, 'cache' => false],
			'commission' => true,
			'formatted_requested_amount' => true,
			'formatted_payout_amount' => true,
			'formatted_charged_amount' => true
		];

		$structure->relations = [
			'User' => [
				'entity' => 'XF:User',
				'type' => self::TO_ONE,
				'conditions' => 'user_id',
				'primary' => true
			],
			'UpdateUser' => [
				'entity' => 'XF:User',
				'type' => self::TO_ONE,
				'conditions' => [['user_id', '=', '$update_user_id']],
				'primary' => true
			],
			'Requisite' => [
				'entity' => Addon::shortName('Requisite'),
				'type' => self::TO_ONE,
				'conditions' => 'requisite_id',
				'primary' => true
			],
			'PaymentSystem' => [
				'entity' => Addon::shortName('PaymentSystem'),
				'type' => self::TO_ONE,
				'conditions' => 'system_id',
				'primary' => true
			],
		];

		$structure->withAliases = [
			'api' => [
				'User',
				'User.api',
				'UpdateUser',
				'UpdateUser.api',
			]
		];

		self::addCurrencyHandlerStructureElements($structure);

		return $structure;
	}

	/**
	 * @return Repository|\PB\WithdrawalQueue\Repository\Withdraw
	 */
	protected function getWithdrawQueueRepo()
	{
		return $this->repository(Addon::shortName('Withdraw'));
	}
}