<?php


namespace PB\WithdrawalQueue\Pub\Controller;


use PB\WithdrawalQueue\Addon;
use PB\WithdrawalQueue\Entity\PaymentSystem;
use PB\WithdrawalQueue\Entity\Requisite as RequisiteEntity;
use XF;
use XF\App;
use XF\ControllerPlugin\Delete;
use XF\Http\Request;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Repository;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\Exception;
use XF\Mvc\Reply\View;

class Requisite extends AbstractBasicController
{
	public function __construct(App $app, Request $request)
	{
		parent::__construct($app, $request);

		$this->setFormatters(Addon::shortName('Requisite%s'), Addon::prefix('requisite_%s'));
		$this->setRoutePrefix('withdraw/requisite');
		$this->setEntityIdentifier(Addon::shortName('Requisite'));
		$this->setPrimaryKey('requisite_id');
	}

	protected function addWithdrawWrapperParams(View $view, $selected)
	{
		$view->setParam('pageSelected', $selected);
		return $view;
	}

	public function actionIndex()
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = XF::visitor();
		if (!$visitor->canViewWithdraws($error))
		{
			return $this->noPermission($error);
		}

		$requisiteRepo = $this->getRequisiteRepo();
		$userRequisites = $requisiteRepo->findRequisites()->forUser($visitor)->fetch();

		$viewParams = [
			'link' => $this->routePrefix,
			'items' => $userRequisites
		];

		$view = $this->view(
			$this->formatView('List'),
			$this->formatTemplate('list'),
			$viewParams
		);

		return $this->addWithdrawWrapperParams($view, 'requisite');
	}

	public function actionPaymentSystemChoose()
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = XF::visitor();
		if (!$visitor->canCreateWithdraws($error))
		{
			return $this->noPermission();
		}

		/** @var \PB\WithdrawalQueue\Repository\PaymentSystem $paymentSystemRepo */
		$paymentSystemRepo = $this->repository(Addon::shortName('PaymentSystem'));
		$paymentSystemFinder = $paymentSystemRepo->findPaymentSystems()->onlyActive();
		$paymentSystems = $paymentSystemFinder->fetch();

		$paymentSystemCount = $paymentSystems->count();
		if ($paymentSystemCount <= 0)
		{
			return $this->error(Addon::phrase('no_payment_systems_configured'));
		}

		$viewParams = [
			'link' => $this->routePrefix,
			'paymentSystems' => $paymentSystems,
		];

		$view = $this->view(
			$this->formatView('Add'),
			$this->formatTemplate('payment_system_chooser'),
			$viewParams
		);

		return $this->addWithdrawWrapperParams($view, 'requisite');
	}

	public function actionPayout(ParameterBag $params)
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = XF::visitor();
		if (!$visitor->canCreateWithdraws($error))
		{
			return $this->noPermission();
		}

		$requisite = $this->assertViewableRequisite($params->requisite_id, 'PaymentSystem');

		/** @var \PB\WithdrawalQueue\Entity\PaymentSystem $paymentSystem */
		$paymentSystem = $requisite->PaymentSystem;
		if (!$paymentSystem || !$paymentSystem->active)
		{
			throw $this->errorException(Addon::phrase('selected_payment_system_is_disabled'));
		}

		if ($this->isPost())
		{
			$creator = $this->setupWithdrawCreate($requisite);
			if (!$creator->validate($errors))
			{
				return $this->error($errors);
			}

			$withdraw = $creator->save();
			$this->finalizeWithdrawCreate();

			return $this->redirect($this->buildLink('withdraw') . $this->buildLinkHash($withdraw->getEntityId()));
		}

		$viewParams = [
			'requisite' => $requisite,
			'link' => $this->routePrefix,
		];

		$view = $this->view(
			$this->formatView('Payout'),
			$this->formatTemplate('payout'),
			$viewParams
		);

		return $this->addWithdrawWrapperParams($view, 'withdraw');
	}

	public function actionGetPayoutSum()
	{
		$input = $this->filter([
			'payment_system_id' => 'uint',
			'sum' => 'unum'
		]);

		$paymentSystem = $this->assertPaymentSystemExists($input['payment_system_id']);

		if (!$input['sum'] || $input['sum'] <= 0)
		{
			return $this->error(\XF::phrase('please_enter_valid_amount'));
		}

		$receiveAmount = $paymentSystem->getActualPayoutAmount($input['sum']);
		$chargeAmount = $paymentSystem->getActualChargeAmount($input['sum'], true);

		$view = $this->view();
		$view->setJsonParams([
			'amount' => number_format($receiveAmount, $paymentSystem->decimal_places),
			'charge_amount' => $chargeAmount
		]);

		return $view;
	}

	protected function setupWithdrawCreate(RequisiteEntity $requisite)
	{
		/** @var \PB\WithdrawalQueue\Service\Withdraw\Creator $creator */
		$creator = $this->service(Addon::shortName('Withdraw\Creator'), $requisite);
		$creator->setAmount($this->filter('sum', 'float'));

		return $creator;
	}

	protected function finalizeWithdrawCreate()
	{
	}

	public function actionAdd()
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = XF::visitor();
		if (!$visitor->canCreateWithdraws($error))
		{
			return $this->noPermission();
		}

		$systemId = $this->filter('system_id', 'uint');
		$system = $this->assertPaymentSystemExists($systemId);

		if ($this->isPost())
		{
			$creator = $this->setupRequisiteCreator($system);
			if (!$creator->validate($errors))
			{
				return $this->error($errors);
			}

			$requisite = $creator->save();
			$this->finalizeRequisiteCreate($creator);

			return $this->redirect($this->buildLink($this->routePrefix) . $this->buildLinkHash($requisite->getEntityId()));
		}

		$viewParams = [
			'link' => $this->routePrefix,
			'paymentSystem' => $system,
			'requisite' => $system->getNewRequisite(),
			'redirect' => $this->getDynamicRedirect()
		];

		$view = $this->view(
			$this->formatView('Add'),
			$this->formatTemplate('add'),
			$viewParams
		);

		return $this->addWithdrawWrapperParams($view, 'requisite');
	}

	protected function setupRequisiteCreator(PaymentSystem $paymentSystem)
	{
		/** @var \PB\WithdrawalQueue\Service\Requisite\Creator $creator */
		$creator = $this->service(Addon::shortName('Requisite\Creator'), $paymentSystem);
		$creator->setWallet($this->filter('wallet', 'str'));

		$customFields = $this->filter('custom_fields', 'array');
		$creator->setCustomFields($customFields);

		return $creator;
	}

	protected function finalizeRequisiteCreate(\PB\WithdrawalQueue\Service\Requisite\Creator $creator)
	{
	}

	public function actionEdit(ParameterBag $params)
	{
		/** @var \PB\WithdrawalQueue\XF\Entity\User $visitor */
		$visitor = XF::visitor();
		if (!$visitor->canCreateWithdraws($error))
		{
			return $this->noPermission();
		}

		$requisite = $this->assertViewableRequisite($params[$this->primaryKey]);

		if ($this->isPost())
		{
			$editor = $this->setupRequisiteEditor($requisite);
			if (!$editor->validate($errors))
			{
				return $this->error($errors);
			}

			$requisite = $editor->save();
			$this->finalizeRequisiteEdit($editor);

			return $this->redirect($this->buildLink($this->routePrefix) . $this->buildLinkHash($requisite->getEntityId()));
		}

		$viewParams = [
			'link' => $this->routePrefix,
			'requisite' => $requisite,
			'redirect' => $this->getDynamicRedirect()
		];

		$view = $this->view(
			$this->formatView('Edit'),
			$this->formatTemplate('edit'),
			$viewParams
		);

		return $this->addWithdrawWrapperParams($view, 'requisite');
	}

	protected function setupRequisiteEditor(RequisiteEntity $requisite)
	{
		/** @var \PB\WithdrawalQueue\Service\Requisite\Editor $editor */
		$editor = $this->service(Addon::shortName('Requisite\Editor'), $requisite);
		$editor->setWallet($this->filter('wallet', 'str'));

		$customFields = $this->filter('custom_fields', 'array');
		$editor->setCustomFields($customFields);

		return $editor;
	}

	protected function finalizeRequisiteEdit(\PB\WithdrawalQueue\Service\Requisite\Editor $editor)
	{
	}

	/**
	 * @param ParameterBag $params
	 * @return XF\Mvc\Reply\AbstractReply
	 * @throws Exception
	 */
	public function actionDelete(ParameterBag $params)
	{
		/** @var RequisiteEntity $requisite */
		$requisite = $this->assertViewableRequisite($params[$this->primaryKey]);

		if (!$requisite->canDelete())
		{
			return $this->noPermission();
		}

		/** @var Delete $plugin */
		$plugin = $this->plugin('XF:Delete');
		return $plugin->actionDelete(
			$requisite,
			$this->buildLink($this->routePrefix . '/delete', $requisite),
			$this->buildLink($this->routePrefix . '/edit', $requisite),
			$this->buildLink($this->routePrefix),
			$requisite->wallet
		);
	}

	/**
	 * @param $id
	 * @param null $with
	 * @param null $phraseKey
	 * @return Entity|PaymentSystem
	 * @throws Exception
	 */
	protected function assertPaymentSystemExists($id, $with = null, $phraseKey = null)
	{
		return $this->assertRecordExists(Addon::shortName('PaymentSystem'), $id, $with, $phraseKey);
	}

	public function assertViewableRequisite($id, $with = null)
	{
		$requisite = $this->assertRequisiteExists($id, $with);

		if (!$requisite->canView())
		{
			throw $this->exception($this->noPermission());
		}

		return $requisite;
	}

	/**
	 * @param $id
	 * @param null $with
	 * @param null $phraseKey
	 * @return Entity|RequisiteEntity
	 * @throws Exception
	 */
	protected function assertRequisiteExists($id, $with = null, $phraseKey = null)
	{
		return $this->assertRecordExists($this->entityIdentifier, $id, $with, $phraseKey);
	}

	/**
	 * @return Repository|\PB\WithdrawalQueue\Repository\Requisite
	 */
	protected function getRequisiteRepo()
	{
		return $this->repository(Addon::shortName('Requisite'));
	}
}