<?php

namespace PB\WithdrawalQueue\Service\PaymentSystem;

use PB\WithdrawalQueue\Entity\PaymentSystem;
use XF;
use XF\App;
use XF\Http\Upload;
use XF\Util\File;

class Icon extends \XF\Service\AbstractService
{
	protected $error = null;

	protected $fileName;

	/**
	 * @var PaymentSystem
	 */
	protected $paymentSystem;

	protected $allowedTypes = [IMAGETYPE_GIF, IMAGETYPE_JPEG, IMAGETYPE_PNG];

	public function __construct(App $app, PaymentSystem $paymentSystem)
	{
		parent::__construct($app);

		$this->setHelpPage($paymentSystem);
	}

	protected function setHelpPage(PaymentSystem $paymentSystem)
	{
		$this->paymentSystem = $paymentSystem;
	}

	public function getError()
	{
		return $this->error;
	}

	public function setImage($fileName)
	{
		if (!$this->validateImage($fileName, $error))
		{
			$this->error = $error;
			$this->fileName = null;
			return false;
		}

		$this->fileName = $fileName;
		return true;
	}

	public function setImageFromUpload(Upload $upload)
	{
		$upload->requireImage();

		if (!$upload->isValid($errors))
		{
			$this->error = reset($errors);
			return false;
		}

		return $this->setImage($upload->getTempFile());
	}

	public function updateImage()
	{
		if (!$this->fileName)
		{
			throw new \LogicException('No source file for image set');
		}

		$paymentSystem = $this->paymentSystem;
		if (!$paymentSystem || !$paymentSystem->exists())
		{
			throw new \LogicException('Help page does not exist');
		}

		$paymentSystem->image_date = XF::$time;

		$sizeMap = $paymentSystem->getImageSizeMap();
		$outputFiles = ['l' => $this->fileName];

		foreach ($sizeMap as $code => $size)
		{
			if (!isset($outputFiles[$code]))
			{
				$newTempFile = $this->loadImageFromFileAndResize($this->fileName, $size);
				if (empty($newTempFile))
				{
					continue;
				}
				$outputFiles[$code] = $newTempFile;
			}
		}

		if (count($outputFiles) != count($sizeMap))
		{
			throw new \RuntimeException('Failed to prepare output files');
		}

		foreach ($outputFiles as $code => $file)
		{
			$dataFile = $paymentSystem->getImageAbstractedPath($code);
			File::copyFileToAbstractedPath($file, $dataFile);
		}

		if ($paymentSystem->hasChanges())
		{
			try
			{
				$paymentSystem->save();
			}
			catch (XF\PrintableException|\Exception $e)
			{
				$this->error = $e;
			}
		}
	}

	public function validateImage($fileName, &$error = null)
	{
		$error = null;

		if (!file_exists($fileName))
		{
			throw new \InvalidArgumentException("File '$fileName' does not exist");
		}
		if (!is_readable($fileName))
		{
			throw new \InvalidArgumentException("File '$fileName' is not readable");
		}

		$imageInfo = filesize($fileName) ? @getimagesize($fileName) : false;
		if (!$imageInfo)
		{
			$error = XF::phrase('provided_file_is_not_valid_image');
			return false;
		}

		$type = $imageInfo[2];
		if (!in_array($type, $this->allowedTypes, true))
		{
			$error = XF::phrase('provided_file_is_not_valid_image');
			return false;
		}

		$width = $imageInfo[0];
		$height = $imageInfo[1];

		if (!$this->app->imageManager()->canResize($width, $height))
		{
			$error = XF::phrase('uploaded_image_is_too_big');
			return false;
		}

		return true;
	}

	/**
	 * @param $fileName
	 * @param $size
	 * @return string|null
	 */
	protected function loadImageFromFileAndResize($fileName, $size)
	{
		$imageManager = $this->app->imageManager();

		if ($image = $imageManager->imageFromFile($fileName))
		{
			$image->resizeShortEdge($size * 2);

			$newTempFile = File::getTempFile();
			if ($newTempFile && $image->save($newTempFile))
			{
				unset($image);
				return $newTempFile;
			}

			return null;
		}

		return null;
	}
}